'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

const createHoc = (hook) => (map) => (component) => {
    const hoc = (props) => {
        const [state, actions] = hook();
        return react.createElement(component, Object.assign({}, props, map(state, actions)));
    };
    hoc.displayName = `WithDakpan(${component.displayName || component.name || 'Unknown'})`;
    return hoc;
};

const createHook = (context) => () => {
    const state = react.useContext(context);
    if (!state) {
        throw Error('Provider not mounted');
    }
    return [
        state.state,
        state.actions
    ];
};

const createProvider = (context, actions, initialState) => ({ children }) => {
    const [state, setState] = react.useState(initialState);
    const mapped = react.useMemo(() => Object.keys(actions).reduce((previous, current) => {
        const execute = (...args) => () => Promise.resolve(actions[current](...args)(state))
            .then((next) => {
            if (next) {
                setState(next);
            }
        });
        const action = (...args) => execute(...args)();
        action.c = execute;
        return Object.assign({}, previous, { [current]: action });
    }, {}), [actions]);
    return react.createElement(context.Provider, {
        value: {
            state,
            actions: mapped
        }
    }, children);
};

const createDakpan = (initialState) => (actions) => {
    const context = react.createContext(undefined);
    const hook = createHook(context);
    return [
        createProvider(context, actions, initialState),
        hook,
        createHoc(hook)
    ];
};

exports.createDakpan = createDakpan;
//# sourceMappingURL=index.js.map
